@@ -1,10 +1,10 @@
 import datetime
-from fast_trade.validate_backtest import validate_backtest
 import pandas as pd
 import re
+import itertools
 
 from .build_data_frame import build_data_frame
-from .run_analysis import analyze_df
+from .run_analysis import apply_logic_to_df
 from .build_summary import build_summary
 
 
@@ -23,39 +23,43 @@ def run_backtest(
             trade_log, dataframe of all the rows where transactions happened
     """
 
-    try:
-        perf_start_time = datetime.datetime.utcnow()
-        new_backtest = prepare_new_backtest(backtest)
-        if ohlcv_path:
-            df = build_data_frame(backtest, ohlcv_path)
+    performance_start_time = datetime.datetime.utcnow()
+    new_backtest = prepare_new_backtest(backtest)
+    if ohlcv_path:
+        df = build_data_frame(new_backtest, ohlcv_path)
 
-        df = apply_backtest_to_dataframe(df, new_backtest)
+    df = apply_backtest_to_df(df, new_backtest)
 
-        if summary:
-            summary, trade_log = build_summary(df, perf_start_time, new_backtest)
-        else:
-            perf_stop_time = datetime.datetime.utcnow()
-            summary = {"test_duration": (perf_stop_time - perf_start_time).total_seconds()}
-            trade_log = None
-
-        return {
-            "summary": summary,
-            "df": df,
-            "trade_df": trade_log,
-            "backtest": new_backtest,
-        }
-    except Exception as e:
-        return {
-            "summary": None,
-            "df": None,
-            "trade_df": None,
-            "backtest": None,
-            "error": e,
-            "backtest_validation": validate_backtest(backtest)
+    if summary:
+        summary, trade_log = build_summary(df, performance_start_time)
+    else:
+        performance_stop_time = datetime.datetime.utcnow()
+        summary = {
+            "test_duration": (
+                performance_stop_time - performance_start_time
+            ).total_seconds()
         }
+        trade_log = None
+
+    return {
+        "summary": summary,
+        "df": df,
+        "trade_df": trade_log,
+        "backtest": new_backtest,
+    }
 
 
 def prepare_new_backtest(backtest):
+    """
+    Parameters
+    ----------
+        backtest, a raw backest object
+
+    Returns
+    -------
+        backtest, a backtest dictionary normalized with defaults
+
+    """
     new_backtest = backtest.copy()
     new_backtest["base_balance"] = backtest.get("base_balance", 1000)
     new_backtest["exit_on_end"] = backtest.get("exit_on_end", False)
@@ -63,22 +67,13 @@ def prepare_new_backtest(backtest):
     new_backtest["trailing_stop_loss"] = backtest.get("trailing_stop_loss", 0)
     new_backtest["any_enter"] = backtest.get("any_enter", [])
     new_backtest["any_exit"] = backtest.get("any_exit", [])
+    new_backtest["lot_size_perc"] = float(backtest.get("lot_size", 1))
+    new_backtest["max_lot_size"] = int(backtest.get("max_lot_size", 0))
 
     return new_backtest
 
 
-def flatten_to_logics(list_of_logics):
-    if len(list_of_logics) < 2:
-        return list_of_logics
-    if isinstance(list_of_logics[0], list):
-        return flatten_to_logics(list_of_logics[0]) + flatten_to_logics(
-            list_of_logics[1:]
-        )
-
-    return list_of_logics[:1] + flatten_to_logics(list_of_logics[1:])
-
-
-def apply_backtest_to_dataframe(df: pd.DataFrame, backtest: dict):
+def apply_backtest_to_df(df: pd.DataFrame, backtest: dict):
     """Processes the frame and adds the resultent rows
     Parameters
     ----------
@@ -90,17 +85,36 @@ def apply_backtest_to_dataframe(df: pd.DataFrame, backtest: dict):
         df, dataframe with with all the actions and backtest processed
     """
 
-    df = process_logic_and_actions(df, backtest)
+    df = process_logic_and_generate_actions(df, backtest)
 
-    df = analyze_df(df, backtest)
+    df = apply_logic_to_df(df, backtest)
 
-    df["aux_perc_change"] = df["total_value"].pct_change() * 100
-    df["aux_change"] = df["total_value"].diff()
+    df["adj_account_value_change_perc"] = df["adj_account_value"].pct_change()
+    df["adj_account_value_change"] = df["adj_account_value"].diff()
 
     return df
 
 
-def process_logic_and_actions(df, backtest):
+def process_logic_and_generate_actions(df: pd.DataFrame, backtest: object):
+    """
+    Parameters
+    ----------
+        df, dataframe with the datapoints (indicators) calculated
+        backtest, backtest object
+
+    Returns
+    -------
+        df, a modified dataframe with the "actions" added
+
+    Explainer
+    ---------
+    In this function, like the name suggests, we process the logic and generate the actions.
+
+    """
+
+    """we need to search though all the logics and find the highest confirmation number
+    so we know how many frames to pass in
+    """
     logics = [
         backtest["enter"],
         backtest["exit"],
@@ -108,8 +122,8 @@ def process_logic_and_actions(df, backtest):
         backtest["any_enter"],
     ]
 
-    max_last_frames = 0
-
+    logics = list(itertools.chain(*logics))
+    max_last_frames = 0  # the we need to keep the
     for logic in logics:
         if len(logic) > 3:
             if logic[3] > max_last_frames:
@@ -122,10 +136,8 @@ def process_logic_and_actions(df, backtest):
             last_frames.insert(0, frame)
             if len(last_frames) >= max_last_frames + 1:
                 last_frames.pop()
-
-            actions.append(
-                determine_action(frame, backtest, max_last_frames, last_frames)
-            )
+            wtf = determine_action(frame, backtest, last_frames)
+            actions.append(wtf)
         df["action"] = actions
     else:
         df["action"] = [determine_action(frame, backtest) for frame in df.itertuples()]
@@ -133,9 +145,7 @@ def process_logic_and_actions(df, backtest):
     return df
 
 
-def determine_action(
-    frame: pd.DataFrame, backtest: dict, max_last_frames=0, last_frames=[]
-):
+def determine_action(frame: pd.DataFrame, backtest: dict, last_frames=[]):
     """processes the actions with the applied logic
     Parameters
     ----------
@@ -154,28 +164,22 @@ def determine_action(
         if frame.close <= frame.trailing_stop_loss:
             return "tsl"
 
-    if take_action(frame, backtest["exit"], max_last_frames, last_frames):
+    if take_action(frame, backtest["exit"], last_frames):
         return "x"
 
-    if take_action(
-        frame, backtest["any_exit"], max_last_frames, last_frames, require_any=True
-    ):
+    if take_action(frame, backtest["any_exit"], last_frames, require_any=True):
         return "ax"
 
-    if take_action(frame, backtest["enter"], max_last_frames, last_frames):
+    if take_action(frame, backtest["enter"], last_frames):
         return "e"
 
-    if take_action(
-        frame, backtest["any_enter"], max_last_frames, last_frames, require_any=True
-    ):
+    if take_action(frame, backtest["any_enter"], last_frames, require_any=True):
         return "ae"
 
     return "h"
 
 
-def take_action(
-    current_frame, logics, max_last_frames, last_frames=[], require_any=False
-):
+def take_action(current_frame, logics, last_frames=[], require_any=False):
     """determines whether to take action based on the logic in the backtest
     Parameters
     ----------
@@ -191,15 +195,36 @@ def take_action(
     results = []
 
     if len(last_frames):
-        for last_frame in last_frames:
-            res = process_single_frame(logics, last_frame, require_any)
-            results.append(res)
+        for logic in logics:
+            if len(logic) > 3:
+                frames = logic[3]
+                lcl_frames = last_frames[:frames]
+                lcl_results = []
 
+                if len(lcl_frames) < frames:
+                    res = False
+                else:
+                    for lcl_frame in lcl_frames:
+                        lcl_res = process_single_frame(
+                            [logic], lcl_frame, require_any=False
+                        )
+                        lcl_results.append(lcl_res)
+                    res = all(lcl_results)
+                results.append(res)
+            else:
+                res = process_single_frame([logic], current_frame, require_any)
+                results.append(res)
     else:
         res = process_single_frame(logics, current_frame, require_any)
         results.append(res)
 
-    return all(results)
+    ret_value = False
+    if len(results):
+        if require_any:
+            ret_value = any(results)
+        else:
+            ret_value = all(results)
+    return ret_value
 
 
 def process_single_frame(logics, row, require_any):
@@ -208,6 +233,7 @@ def process_single_frame(logics, row, require_any):
     return_value = False
     for logic in logics:
         res = process_single_logic(logic, row)
+        # print("row: ", row)
         results.append(res)
 
     if len(results):
@@ -219,6 +245,22 @@ def process_single_frame(logics, row, require_any):
     return return_value
 
 
+def process_single_logic(logic, row):
+    val0 = clean_field_type(logic[0], row=row)
+    val1 = clean_field_type(logic[2], row=row)
+
+    if logic[1] == ">":
+        return_value = bool(val0 > val1)
+    if logic[1] == "<":
+        return_value = bool(val0 < val1)
+    if logic[1] == "=":
+        return_value = bool(val0 == val1)
+    if logic[1] == "!=":
+        return_value = bool(val0 != val1)
+
+    return return_value
+
+
 def clean_field_type(field, row=None):
     """Determines the value of what to run the logic against.
         This might be a calculated value from the current row,
@@ -243,26 +285,15 @@ def clean_field_type(field, row=None):
             return int(field)
         if re.match(r"^-?\d+(?:\.\d+)$", field):  # if its a string in a float
             return float(field)
-    elif isinstance(field, int) or isinstance(field, float):
+
+    if type(field) is bool:
+        return field
+
+    if isinstance(field, int) or isinstance(field, float):
+
         return field
 
     if row:
         return row[field]
 
     return row
-
-
-def process_single_logic(logic, row):
-    val0 = clean_field_type(logic[0], row=row)
-    val1 = clean_field_type(logic[2], row=row)
-
-    if logic[1] == ">":
-        return_value = bool(val0 > val1)
-    if logic[1] == "<":
-        return_value = bool(val0 < val1)
-    if logic[1] == "=":
-        return_value = bool(val0 == val1)
-    if logic[1] == "!=":
-        return_value = bool(val0 != val1)
-
-    return return_value
